VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cTextBox"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
' ************************************************************************
' *
' *   Advanced Textbox
' *   Erweiterte Textbox mit Klasse
' *
' *   Das Klassenmodul cTextBox besitzt folgende Features:
' *      - AutoSelText bei Fokuserhalt
' *      - Wechseln zum nchsten Objekt mit Return
' *      - Demarkieren des Textes mit F2 (MS-Office like)
' *      - Wiederherstellung des Original Text (STRG-O)
' *      - Textbox auf ReadOnly setzen (Textfarbe Rot)
' *      - Textbox nur numerische Werte (Textfarbe Blau)
' *      - GRO und klein Schreibweise
' *      - Lngenbegrenzung (Optional = 256 Zeichen)
' *      - MouseOver Hover-Effekt
' *
' *   Juli 2003 - Roland Wutzke     mailto:rwutzke@t-online.de
' *
' ************************************************************************

Option Explicit

' Bentigte APIs
Private Declare Function GetWindowLong Lib "user32" _
  Alias "GetWindowLongA" ( _
  ByVal hwnd As Long, _
  ByVal nIndex As Long) As Long

Private Declare Function SetWindowLong Lib "user32" _
  Alias "SetWindowLongA" ( _
  ByVal hwnd As Long, _
  ByVal nIndex As Long, _
  ByVal dwNewLong As Long) As Long

Private Declare Function SendMessage Lib "user32" _
  Alias "SendMessageA" ( _
  ByVal hwnd As Long, _
  ByVal wMsg As Long, _
  ByVal wParam As Long, _
  lParam As Any) As Long

Private Declare Function SetCapture Lib "user32" _
  (ByVal hwnd As Long) As Long
   
Private Declare Function ReleaseCapture Lib "user32" _
  () As Long

' Bentigte Aufzhlung
Public Enum eCaseMode
  NormalCase = 0
  UpperCase = 1
  LowerCase = 2
End Enum

' Bentigte Konstanten
Private Const GWL_STYLE As Long = (-16)
Private Const ES_NUMBER As Long = &H2000
Private Const ES_UPPERCASE As Long = &H8&
Private Const ES_LOWERCASE As Long = &H10&
Private Const EM_SETREADONLY As Long = &HCF

' TextBox mit Ereignisse
Public WithEvents TxtBox As TextBox
Attribute TxtBox.VB_VarHelpID = -1

' Private Klassen Variable
Private defStyle As Long
Private oldStyle As Long
Private oldColor As OLE_COLOR
Private oldBold As Boolean
Private oText As String
Private MaxLen As Long
Private isMouseOver As Boolean
Private mvarisReadOnly As Boolean
Private mvarisNumeric As Boolean
Private mvarMaxTextLen As Long
Private mvarCaseMode As Long
Private mvarcanHover As Boolean
Private Sub Class_Initialize()
  MaxTextLen = 256
  isMouseOver = False
  canHover = True
  oldStyle = 0
End Sub

Private Sub TxtBox_GotFocus()
  ' Die Textbox mit dem neuen Style belegen.
  With TxtBox
    defStyle = GetWindowLong(.hwnd, GWL_STYLE)
    If oldStyle = 0 Then oldStyle = defStyle
    MaxLen = MaxTextLen
    oText = .Text
    .SelStart = 0
    .SelLength = Len(.Text)
  End With
End Sub

Private Sub TxtBox_KeyDown(KeyCode As Integer, Shift As Integer)
  With TxtBox
    Select Case KeyCode
      ' Prfen ob F2 gedrckt wurde, dann den Text
      ' demarkieren und an das Textende springen.
      ' MS-Office like...
       Case vbKeyF2 And Shift = 0
        If .SelLength > 0 Then
          .SelLength = 0
          .SelStart = Len(.Text)
        End If
         
      ' Prfen ob CTRL-O gedrckt, dann den Original
      ' Text wiederherstellen.
      Case vbKeyO And Shift = 2
        If Not isReadOnly Then .Text = oText
         
      ' Alle anderen Zeichen an die Textbox
      ' durchlassen.
      Case Else
         
    End Select
  End With
End Sub

Private Sub TxtBox_KeyPress(KeyAscii As Integer)
  ' Prfen ob RETURN gedrckt wurde, dann mit
  ' SendKeys {TAB} die Textbox verlassen.
  If KeyAscii = 13 Then
    KeyAscii = 0
    SendKeys "{TAB}", True
    Exit Sub
  End If
   
  ' Prfen ob die maximale Textlnge erreicht ist.
  If KeyAscii <> vbKeyBack Then
    If Len(TxtBox.Text) >= MaxLen Then
      KeyAscii = 0
    End If
  End If
End Sub

Private Sub TxtBox_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
  ' Hover Effekt, wenn die Maus ber die Textbox fhrt.
  If canHover Then
    Dim l As Long
    
    With TxtBox
      If X >= 0 And X <= .Width And _
        Y >= 0 And Y <= .Height Then
            
        l = SetCapture(.hwnd)
        If Not isMouseOver Then
          oldColor = .ForeColor
          oldBold = .FontBold
          isMouseOver = True
        End If
            
        .FontUnderline = True
        .FontBold = True
        .ForeColor = vbBlue
      
      Else
        l = ReleaseCapture()
        .FontUnderline = False
        .ForeColor = oldColor
        .FontBold = oldBold
        isMouseOver = False
      End If
    End With
  End If
End Sub

Public Property Let canHover(ByVal vData As Boolean)
  mvarcanHover = vData
End Property

Public Property Get canHover() As Boolean
  canHover = mvarcanHover
End Property

Public Property Let CaseMode(ByVal vData As eCaseMode)
  mvarCaseMode = vData
  If oldStyle = 0 Then TxtBox_GotFocus
  
  With TxtBox
    Select Case mvarCaseMode
      Case eCaseMode.NormalCase
        Call SetWindowLong(.hwnd, GWL_STYLE, oldStyle)
      Case eCaseMode.LowerCase
        Call SetWindowLong(.hwnd, GWL_STYLE, _
          oldStyle Or ES_LOWERCASE)
        .Text = LCase$(.Text)
      Case eCaseMode.UpperCase
        Call SetWindowLong(.hwnd, GWL_STYLE, _
          oldStyle Or ES_UPPERCASE)
        .Text = UCase$(.Text)
    End Select
  End With
End Property

Public Property Get CaseMode() As eCaseMode
  CaseMode = mvarCaseMode
End Property

Public Property Let isNumeric(ByVal vData As Boolean)
  mvarisNumeric = vData
  If oldStyle = 0 Then TxtBox_GotFocus
   
  With TxtBox
    .ForeColor = IIf(mvarisNumeric = True, vbBlue, oldColor)
    If mvarisNumeric Then
      Call SetWindowLong(.hwnd, GWL_STYLE, _
        oldStyle Or ES_NUMBER)
    Else
      Call SetWindowLong(.hwnd, GWL_STYLE, oldStyle)
    End If
  End With
End Property

Public Property Get isNumeric() As Boolean
  isNumeric = mvarisNumeric
End Property

Public Property Let isReadOnly(ByVal vData As Boolean)
  mvarisReadOnly = vData
  If oldStyle = 0 Then TxtBox_GotFocus
  
  With TxtBox
    .ForeColor = IIf(mvarisReadOnly = True, vbRed, oldColor)
    Call SendMessage(.hwnd, EM_SETREADONLY, _
      mvarisReadOnly, ByVal 0&)
  End With
End Property

Public Property Get isReadOnly() As Boolean
  isReadOnly = mvarisReadOnly
End Property

Public Property Let MaxTextLen(ByVal vData As Long)
  mvarMaxTextLen = vData
End Property

Public Property Get MaxTextLen() As Long
  MaxTextLen = mvarMaxTextLen
End Property

